# This file is part of the standard setup for testthat.
# It is recommended that you do not modify it.
#
# Where should you do your testing? Inside tests/testthat/ directory.

library(testthat)
library(rwetools)

test_check("rwetools")
