% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers_table1.R
\name{levels_excl_zero}
\alias{levels_excl_zero}
\title{Get non-baseline factor levels}
\usage{
levels_excl_zero(f)
}
\arguments{
\item{f}{A factor.}
}
\value{
Character vector of non-baseline levels.
}
\description{
Returns all factor levels except those commonly used as a baseline
(e.g., "0", "No", "None", "FALSE").
}
\keyword{internal}
