% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers_ps_psweights.R
\name{add_readme_sheet}
\alias{add_readme_sheet}
\title{Add a README sheet to an openxlsx workbook}
\usage{
add_readme_sheet(wb, readme_text, verbose = TRUE)
}
\arguments{
\item{wb}{An openxlsx workbook object}

\item{readme_text}{Character string (may contain newlines)}

\item{verbose}{Logical. Print progress messages (default TRUE).}
}
\value{
Called for its side effect (adds a sheet to \code{wb}). Returns
\code{NULL} invisibly.
}
\description{
Add a README sheet to an openxlsx workbook
}
\keyword{internal}
